import uuid

from django.core.exceptions import ValidationError
from django.db.models import BooleanField, DateTimeField
from django.db.models import UUIDField as _UUIDField
from django.utils.translation import gettext_lazy as _


class AutoCreatedField(DateTimeField):
    """
    AutoCreatedField

    By default, sets auto_now_add=True
    """

    def __init__(self, *args, **kwargs):
        kwargs.setdefault("auto_now_add", True)
        super().__init__(*args, **kwargs)


class AutoModifiedField(DateTimeField):
    """
    AutoModifiedField

    By default, auto_now=True
    Sets value to now every time the object is saved.
    """

    def __init__(self, *args, **kwargs):
        kwargs.setdefault("auto_now", True)
        super().__init__(*args, **kwargs)

    def pre_save(self, model_instance, add):
        if not getattr(model_instance, "update_modified", True):
            return getattr(model_instance, self.attname)
        return super().pre_save(model_instance, add)


class UUIDFieldMixin:
    """UUIDFieldMixin sets the default value."""

    DEFAULT_MAX_LENGTH = 36

    def __init__(self, verbose_name=None, primary_key=True, version=4, editable=False, *args, **kwargs):
        kwargs.setdefault("max_length", self.DEFAULT_MAX_LENGTH)
        kwargs.setdefault("primary_key", primary_key)
        kwargs.setdefault("editable", editable)
        kwargs.setdefault("default", self._get_default(version))
        super().__init__(verbose_name=verbose_name, *args, **kwargs)

    @classmethod
    def _get_default(cls, version):

        if version == 2:
            raise ValidationError(_("UUID version 2 is not supported."))

        if version < 1 or version > 5:
            raise ValidationError(_("UUID version is not valid."))

        if version == 1:
            return uuid.uuid1
        elif version == 3:
            return uuid.uuid3
        elif version == 4:
            return uuid.uuid4
        elif version == 5:
            return uuid.uuid5


class UUIDField(UUIDFieldMixin, _UUIDField):
    """UUID field to use as primary key of model.

    By default, sets primary_key=True, version=4, editable=False
    """


class AutoActiveField(BooleanField):
    """
    AutoActiveField

    By default, sets default=True
    """

    def __init__(self, verbose_name=_("Is active?"), default=True, *args, **kwargs):
        kwargs.setdefault("default", default)
        super().__init__(verbose_name, *args, **kwargs)



from django.db.models import BooleanField, DateTimeField, Manager, Model
from django.utils import timezone
from django.utils.translation import gettext_lazy as _

from core_helpers.db.fields import (
    AutoActiveField,
    AutoCreatedField,
    AutoModifiedField,
    UUIDField,
)
from core_helpers.db.managers import (
    ActiveDeletableManager,
    ActiveManager,
    DeletableManager,
)


class UUIDModel(Model):
    """Abstract model with `uuid` id as primary key."""

    id = UUIDField(_("ID"))

    class Meta:
        abstract = True


class ActiveModel(Model):
    """Abstract `Active UUID` model."""

    is_active = AutoActiveField()

    objects = Manager()
    active = ActiveManager()

    class Meta:
        abstract = True


class TimeStampedModel(Model):
    """Abstract `Created` model."""

    created_at = AutoCreatedField(_("Created at"))
    modified_at = AutoModifiedField(_("Modified at"))

    def save(self, *args, **kwargs):
        self.update_modified = kwargs.pop("update_modified", getattr(self, "update_modified", True))
        update_fields = kwargs.get("update_fields", None)
        if update_fields:
            kwargs["update_fields"] = set(update_fields).union({"modified_at"})

        super().save(*args, **kwargs)

    class Meta:
        abstract = True


class DeletableModel(Model):
    """Abstract `Deletable` model."""

    is_deleted = BooleanField(_("Is deleted"), default=False)
    deleted_at = DateTimeField(_("Deleted at"), null=True, blank=True, editable=False)

    objects = DeletableManager()
    all_objects = Manager()

    class Meta:
        abstract = True

    def delete(self, using=None, keep_parents=False, force=False):
        """Delete the current instance."""
        if force:
            return super().delete(using=using, keep_parents=keep_parents)
        else:
            self.deleted_at = timezone.now()
            self.save(using=using, update_fields=("is_deleted", "deleted_at"))


class DeletableUUIDModel(DeletableModel, UUIDModel):
    """Abstract `Deletable, UUID` mixin model."""

    class Meta:
        abstract = True


class ActiveUUIDModel(ActiveModel, UUIDModel):
    """Abstract `Active UUID` mixin model."""

    class Meta:
        abstract = True


class ActiveTimeStampedModel(ActiveModel, TimeStampedModel):
    """Abstract `Active TimeStamped` mixin model."""

    class Meta:
        abstract = True


class ActiveDeletableModel(ActiveModel, DeletableModel):
    """Abstract `Active Deletable` mixin model."""

    objects = DeletableManager()
    active = ActiveDeletableManager()
    all_objects = Manager()

    class Meta:
        abstract = True


class ActiveTimeStampedUUIDModel(ActiveModel, TimeStampedModel, UUIDModel):
    """Abstract `Active TimeStamped UUID` mixin model."""

    class Meta:
        abstract = True


class ActiveDeletableUUIDModel(ActiveDeletableModel, UUIDModel):
    """Abstract `Active Deletable` mixin model."""

    class Meta:
        abstract = True


class ActiveTimeStampedDeletableModel(TimeStampedModel, ActiveDeletableModel):
    """Abstract `Active, TimeStamped, Deletable` mixin model."""

    class Meta:
        abstract = True


class ActiveTimeStampedDeletableUUIDModel(TimeStampedModel, ActiveDeletableModel, UUIDModel):
    """Abstract `Active, TimeStamped, Deletable, UUID` mixin model."""

    class Meta:
        abstract = True


class TimeStampedDeletableModel(TimeStampedModel, DeletableModel):
    """Abstract `TimeStamped, Deletable` mixin model."""

    class Meta:
        abstract = True


class TimeStampedUUIDModel(TimeStampedModel, UUIDModel):
    """Abstract `TimeStamped, UUIDModel` mixin model."""

    class Meta:
        abstract = True


class TimeStampedDeletableUUIDModel(TimeStampedModel, DeletableModel, UUIDModel):
    """Abstract `TimeStamped, Deletable, UUID` mixin model."""

    class Meta:
        abstract = True



from django.db.models.manager import Manager
from django.db.models.query import QuerySet
from django.utils import timezone


class DeletableQuerySetMixin:
    """Custom `Deletable` QuerySet Mixin for DeletableModel."""

    def delete(self, force: bool = False):
        """Delete the objects from current QuerySet (sets its ``is_deleted`` field to True). """
        if force:
            """Hard delete the objects if `force` param is True"""
            return super().delete()
        else:
            return self.count(), self.update(is_deleted=True, deleted_at=timezone.now())  # noqa


class DeletableQuerySet(DeletableQuerySetMixin, QuerySet):
    """Custom `Deletable` QuerySet for `DeletableManager`."""


class DeletableManagerMixin:

    _queryset_class = DeletableQuerySet

    def get_queryset(self):
        """Returns custom queryset with filter `is_deleted=False`."""

        kwargs = {"model": self.model, "using": self._db}
        if hasattr(self, "_hints"):
            kwargs["hints"] = self._hints

        return self._queryset_class(**kwargs).filter(is_deleted=False)


class GetOrNoneManagerMixin:
    def get_or_none(self, **kwargs):
        """Return model instance or None by `**kwargs`."""
        try:
            return self.get(**kwargs)
        except self.model.DoesNotExist:
            return None


class ActiveManagerMixin:
    def get_queryset(self):
        """Return custom queryset with filter `is_active=True`."""
        return super().get_queryset().filter(is_active=True)


class DeletableManager(DeletableManagerMixin, GetOrNoneManagerMixin, Manager):
    """Custom manager that returns records with an `is_deleted` flag set as `False`."""


class ActiveManager(ActiveManagerMixin, GetOrNoneManagerMixin, Manager):
    """Custom manager  that Return records with an `is_active` flag set as `True`."""


class ActiveDeletableManager(DeletableManagerMixin, ActiveManagerMixin, GetOrNoneManagerMixin, Manager):
    """Custom manager  records with an `is_active` flag set as `True` and `is_deleted` flag as `False`."""



from .core import *

__all__ = [
    "UUIDModel",
    "ActiveModel",
    "TimeStampedModel",
    "DeletableModel",
    "DeletableUUIDModel",
    "ActiveUUIDModel",
    "ActiveTimeStampedModel",
    "ActiveDeletableModel",
    "ActiveTimeStampedUUIDModel",
    "ActiveDeletableUUIDModel",
    "ActiveTimeStampedDeletableModel",
    "ActiveTimeStampedDeletableUUIDModel",
    "TimeStampedUUIDModel",
    "TimeStampedDeletableModel",
    "TimeStampedDeletableUUIDModel",
]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['core_helpers',
 'core_helpers.api',
 'core_helpers.api.filters',
 'core_helpers.api.serializers',
 'core_helpers.api.views',
 'core_helpers.db',
 'core_helpers.db.fields',
 'core_helpers.db.managers',
 'core_helpers.db.models']

package_data = \
{'': ['*']}

install_requires = \
['environs>=9.3.1,<10.0.0', 'typing-extensions>=3.7.4,<4.0.0']

extras_require = \
{'docs': ['sphinx>=3.4.3,<4.0.0', 'sphinx-rtd-theme>=0.5.1,<0.6.0'],
 'test': ['requests>=2.25.1,<3.0.0',
          'requests-mock>=1.8.0,<2.0.0',
          'pytest-django>=4.1.0,<5.0.0',
          'pytest>=6.2.2,<7.0.0',
          'codecov>=2.1.11,<3.0.0',
          'coverage>=5.4,<6.0',
          'freezegun>=1.1.0,<2.0.0',
          'pytest-cov>=2.11.1,<3.0.0',
          'pytest-pythonpath>=0.7.3,<0.8.0']}

setup_kwargs = {
    'name': 'django-core-helpers',
    'version': '0.1.3',
    'description': 'Core helpers for Django and Django Rest Framework',
    'long_description': '# Django Core Helpers\n\n[![Python Version](https://img.shields.io/badge/python-3.8-blue)](https://www.python.org/)\n[![codecov](https://codecov.io/gh/onufrienkovi/django-core-helpers/branch/master/graph/badge.svg)](https://codecov.io/gh/onufrienkovi/django-core-helpers)\n[![wemake-python-styleguide](https://img.shields.io/badge/style-wemake-000000.svg)](https://github.com/wemake-services/wemake-python-styleguide)\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n\n\n## Quickstart\n\nInstall\n\n```console\npip install django-core-helpers\n```\n\n## Prerequisites\n\nYou will need:\n\n- `python3.8` (see `pyproject.toml` for full version)\n- `django` with version `3.0`\n\n\n## Development\n\nWhen developing locally, we use:\n\n- [`editorconfig`](http://editorconfig.org/) plugin (**required**)\n- [`poetry`](https://github.com/python-poetry/poetry) (**required**)\n- [`pyenv`](https://github.com/pyenv/pyenv) (**required**)  \n- `pycharm 2017+` or `vscode`\n\n\n## Documentation\n\nFull documentation is available here: [`docs/`](docs).\n',
    'author': 'Vyacheslav Onufrienko',
    'author_email': 'onufrienkovi@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/onufrienkovi/django-core-helpers',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
